/*
 * Copyright (c) 2017 Linaro Limited
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <device.h>
#include <devicetree.h>
#include <drivers/gpio.h>
#include <sys/util.h>
#include <sys/printk.h>
#include <inttypes.h>
#include <drivers/pwm.h>
#include <sys/__assert.h>
#include <string.h>
#include <dk_buttons_and_leds.h>
#include <drivers/sensor.h>

#include "buzzer.h"
#include "ui.h" 

/* size of stack area used by each thread */
#define STACKSIZE 2048

/* scheduling priority used by each thread */
#define PRIORITY 7
#define SLEEP_MS 500
#define SLEEP_MS_SENSOR 5000
/*FIFO*/
struct printk_data_t {
	void *fifo_reserved; /* 1st word reserved for use by fifo */
	char* conp;
        bool sensors;
        int* val_sensors;
};

K_FIFO_DEFINE(printk_fifo);


void print(char* str, bool sensors, int* val){
    struct printk_data_t tx_data = { .conp = str, .sensors=sensors, .val_sensors=val };
    size_t size = sizeof(struct printk_data_t);
    char *mem_ptr = k_malloc(size);
    __ASSERT_NO_MSG(mem_ptr != 0);

    memcpy(mem_ptr, &tx_data, size);

    k_fifo_put(&printk_fifo, mem_ptr);

    k_msleep(SLEEP_MS);
    
}


/**@brief Callback for button events from the DK buttons and LEDs library. */
static void button_handler(uint32_t button_states, uint32_t has_changed) {
        print("button",0,0);
        printk("Button Pressed\n");
        ui_buzzer_set_frequency(100, 20);
        k_msleep(SLEEP_MS);
        ui_buzzer_set_frequency(0, 0);
}

void blink()
{
	int cnt = 0;
        /*led*/
	int err;

	err = dk_leds_init();
	if (err) {
		printk("Could not initialize leds, err code: %d\n", err);
	}

        /*button*/
        err = dk_buttons_init(button_handler);
        if (err) {
                 printk("Could not initialize leds, err code: %d\n", err);
        }
        
        /*buzzer*/
        err = ui_buzzer_init();
	if (err) {
		printk("Could not enable buzzer, err code: %d\n", err);
	}
        uint32_t leds_on_mask = DK_LED1_MSK;
	while (1 && !err) {
		err = dk_set_leds_state(leds_on_mask, DK_ALL_LEDS_MSK);
                if (err) {
                        printk("Could not set leds state, err code: %d\n", err);
                }
                if (cnt % 2) {
                         leds_on_mask &= ~DK_ALL_LEDS_MSK;
                         leds_on_mask |= DK_LED1_MSK;
                         print("green",0,0);
                }
                else{
                         leds_on_mask &= ~DK_ALL_LEDS_MSK;
                         leds_on_mask |= DK_LED2_MSK;
                         print("red",0,0);
                }
                k_msleep(SLEEP_MS);
                cnt++;              
	}
}

void get_env_sensors_value(void){
        const struct device *env_sensor = device_get_binding(DT_LABEL(DT_INST(0, bosch_bme680)));
	struct sensor_value temp, press, humidity, gas_res;
        int res[4];
        while (1) {
                k_msleep(SLEEP_MS_SENSOR);
		sensor_sample_fetch(env_sensor);
		sensor_channel_get(env_sensor, SENSOR_CHAN_AMBIENT_TEMP, &temp);
		sensor_channel_get(env_sensor, SENSOR_CHAN_PRESS, &press);
		sensor_channel_get(env_sensor, SENSOR_CHAN_HUMIDITY, &humidity);
		sensor_channel_get(env_sensor, SENSOR_CHAN_GAS_RES, &gas_res);
                
                res[0] = temp.val1;
                res[1] = press.val1;
                res[2] = humidity.val1;
                res[3] = gas_res.val1;
                print("Sensors",1,res);
	}

}

void uart_out(void)
{
	while (1) {
		struct printk_data_t *rx_data = k_fifo_get(&printk_fifo,
							   K_FOREVER);
		if(rx_data->sensors){

                      printk("[Info] %s \n T: %d C, P: %d hPa, H: %d, G: %d ohm \n",
                                      rx_data->conp, rx_data->val_sensors[0], rx_data->val_sensors[1],
                                      rx_data->val_sensors[2], rx_data->val_sensors[3]);
                }
                else{
                      printk("[Info] %s \n", rx_data->conp);
                }
		k_free(rx_data);
	}
}


K_THREAD_DEFINE(blink_id, STACKSIZE, blink, NULL, NULL, NULL,
		PRIORITY, 0, 0);
K_THREAD_DEFINE(env_sensors_id, STACKSIZE, get_env_sensors_value, NULL, NULL, NULL,
		PRIORITY, 0, 0);
K_THREAD_DEFINE(uart_out_id, STACKSIZE, uart_out, NULL, NULL, NULL,
		PRIORITY, 0, 0);
